#############################################################################
#
# zzLocale - Zope Locale Support Product
# Copyright (C) 2000 Digital Garage Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#############################################################################

""" zzLocale Zope patcher """

import Products
import urllib
import zzLocale
import os
import cPickle

from AccessControl.User import reqattr, BasicUserFolder
from AccessControl.Role import RoleManager
from App.special_dtml import HTMLFile
from OFS import DTMLDocument, DTMLMethod
from Products.ZSQLMethods import SQL
from App.Management import Tabs
from App.FindHomes import SOFTWARE_HOME

# manage_locales contains buttons locales. These strings must match the
# corresponding button text in the DTML templates for a given language

f = open(os.path.join(os.path.split(__file__)[0],'buttons.dft'), 'r')
manage_locales = cPickle.load(f)
f.close()

# manage_options_locales contains tabs locales.
# DEV NOTE: any gain in unifying manage_locales and manage_options_locales?

f = open(os.path.join(os.path.split(__file__)[0],'tabs.dft'), 'r')
manage_options_locales = cPickle.load(f)
f.close()

locales_dir=os.path.join(SOFTWARE_HOME, 'locales')
langs=zzLocale.find_langs(locales_dir)
for lang in langs:
    buttonsFileName = os.path.join(locales_dir, lang, 'buttons')
    if os.path.isfile(buttonsFileName):
	buttonsFile = open(buttonsFileName, 'r')
        buttonsLocales = cPickle.load(buttonsFile)
	buttonsFile.close()
        for button, text in buttonsLocales.items():
	    manage_locales[button][lang] = text
    tabsFileName = os.path.join(locales_dir, lang, 'tabs')
    if os.path.isfile(tabsFileName):
	tabsFile = open(tabsFileName, 'r')
        tabsLocales = cPickle.load(tabsFile)
	tabsFile.close()
        for tab, text in tabsLocales.items():
	    manage_options_locales[tab][lang] = text

#
# Patch the appropriate objects...
#

def patch():
    """zzLocale Zope classes patcher"""

    BasicUserFolder.manage_users = manage_users
    RoleManager.manage_defined_roles = manage_defined_roles
    RoleManager.manage_access = manage_access
    setattr(RoleManager, 'normal_manage_access', HTMLFile(os.path.join('AccessControl', 'access')))
    setattr(RoleManager, 'method_manage_access', HTMLFile(os.path.join('AccessControl', 'methodAccess')))

    DTMLDocument.addDTMLDocument = addDTMLDocument
    Products.OFSP._m['addDTMLDocument'] = addDTMLDocument

    DTMLMethod.addDTMLMethod = addDTMLMethod
    Products.OFSP._m['addDTMLMethod'] = addDTMLMethod

    SQL.manage_addZSQLMethod = manage_addZSQLMethod
    Products.OFSP._m['manage_addZSQLMethod'] = manage_addZSQLMethod

    Tabs.localized_manage_options__roles__=None
    Tabs.localized_manage_options = localized_manage_options

    # Size changes buttons in DTMLDocuments and DTMLMethods 
    for k,v in manage_locales['Taller'].items():
	if k != 'en': DTMLMethod.DTMLMethod._size_changes[v] = (5,0)
    for k,v in manage_locales['Shorter'].items():
	if k != 'en': DTMLMethod.DTMLMethod._size_changes[v] = (-5,0)
    for k,v in manage_locales['Wider'].items():
	if k != 'en': DTMLMethod.DTMLMethod._size_changes[v] = (0,5)
    for k,v in manage_locales['Narrower'].items():
	if k != 'en': DTMLMethod.DTMLMethod._size_changes[v] = (0,-5)


###########################################
#                PATCHES                  #
###########################################


# 
# class AccessControl.User.BasicUserFolder
#

def manage_users(self,submit=None,REQUEST=None,RESPONSE=None):
    """ """
    if submit in manage_locales['Add...'].values():
        return self._add_User(self, REQUEST)

    if submit in manage_locales['Edit'].values():
        try:    user=self.getUser(reqattr(REQUEST, 'name'))
        except: return MessageDialog(
                title  ='Illegal value',
                message='The specified user does not exist',
                action ='manage_main')
        return self._editUser(self,REQUEST,user=user,password=user.__)

    if submit in manage_locales['Add'].values():
        name    =reqattr(REQUEST, 'name')
        password=reqattr(REQUEST, 'password')
        confirm =reqattr(REQUEST, 'confirm')
        roles   =reqattr(REQUEST, 'roles')
        domains =reqattr(REQUEST, 'domains')
        return self._addUser(name,password,confirm,roles,domains,REQUEST)

    if submit in manage_locales['Change'].values():
        name    =reqattr(REQUEST, 'name')
        password=reqattr(REQUEST, 'password')
        confirm =reqattr(REQUEST, 'confirm')
        roles   =reqattr(REQUEST, 'roles')
        domains =reqattr(REQUEST, 'domains')
        return self._changeUser(name,password,confirm,roles,
                                domains,REQUEST)

    if submit in manage_locales['Delete'].values():
        names=reqattr(REQUEST, 'names')
        return self._delUsers(names,REQUEST)

    return self._mainUser(self, REQUEST)

#
# class AccessControl.Role.RoleManager
#

def manage_defined_roles(self,submit=None,REQUEST=None):
    """Called by management screen."""

    if submit in manage_locales['Add Role'].values():
        role=reqattr(REQUEST, 'role')
        return self._addRole(role, REQUEST)

    if submit in manage_locales['Delete Role'].values():
        roles=reqattr(REQUEST, 'roles')
        return self._delRoles(roles, REQUEST)

def manage_access(
    trueself, self, REQUEST, **kw):
    "Return an interface for making permissions settings"
    if self._isBeingUsedAsAMethod():
        return apply(self.method_manage_access,(trueself, REQUEST), kw)
    else:
        return apply(self.normal_manage_access,(trueself, REQUEST), kw)



#
# module OFS.DTMLDocument
#

def addDTMLDocument(self, id, title='', file='', REQUEST=None, submit=None):
    """Add a DTML Document object with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=DTMLDocument.default_dd_html
    ob=DTMLDocument.DTMLDocument(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit in manage_locales['Add and Edit'].values(): u="%s/%s" % (u,urllib.quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

#
# module OFS.DTMLMethod
#

def addDTMLMethod(self, id, title='', file='', REQUEST=None, submit=None):
    """Add a DTML Method object with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=DTMLMethod.default_dm_html
    ob=DTMLMethod.DTMLMethod(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit in manage_locales['Add and Edit'].values(): u="%s/%s" % (u,urllib.quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

#
# module Products.ZSQLMethods.SQL
#

def manage_addZSQLMethod(self, id, title,
                                connection_id, arguments, template,
                                REQUEST=None, submit=None):
    """Add an SQL Method

    The 'connection_id' argument is the id of a database connection
    that resides in the current folder or in a folder above the
    current folder.  The database should understand SQL.

    The 'arguments' argument is a string containing an arguments
    specification, as would be given in the SQL method cration form.

    The 'template' argument is a string containing the source for the
    SQL Template.
    """
    self._setObject(id, SQL(id, title, connection_id, arguments, template))
    if REQUEST is not None:
        u=REQUEST['URL1']
        if submit in manage_locales['Add and Edit'].values():
            u="%s/%s/manage_main" % (u,id)
        elif submit in manage_locales['Add and Test'].values():
            u="%s/%s/manage_testForm" % (u,id)
        else:
            u=u+'/manage_main'

        REQUEST.RESPONSE.redirect(u)
    return ''


#
# class App.Manager.Tabs
#

def localized_manage_options(self, REQUEST=None):

    if REQUEST is None and hasattr(self, 'aq_acquire'):
        try: REQUEST=self.aq_acquire('REQUEST')
        except: pass
    try: user=REQUEST['AUTHENTICATED_USER']
    except: user=None

    enResult = Tabs.filtered_manage_options(self, REQUEST)

    langs = zzLocale.buildLangList(REQUEST)
    localeResult = []

    for d in enResult:
	ld = d.copy()
        label = ld.get('label', None)
        try:
            for lang in langs:
                if manage_options_locales[label].has_key(lang):
                    ld['label'] = manage_options_locales[label][lang]
                    break
        except:
	    pass
        localeResult.append(ld)

    return localeResult
